use super::macros::ref_getter;
use super::macros::simple_getter;
use super::Element;
use super::GetStandardProperties;
use super::Object;
use super::StandardProperties;

pub type PriorityCookie = u8;
pub type HeadlineLevel = u16;

#[derive(Debug)]
pub struct Document<'s> {
    pub(crate) source: &'s str,
    pub(crate) category: Option<&'s str>,
    pub(crate) zeroth_section: Option<Section<'s>>,
    pub(crate) children: Vec<Heading<'s>>,
}

#[derive(Debug)]
pub struct Heading<'s> {
    pub source: &'s str,
    pub level: HeadlineLevel,
    pub todo_keyword: Option<(TodoKeywordType, &'s str)>,
    pub priority_cookie: Option<PriorityCookie>,
    pub title: Vec<Object<'s>>,
    pub tags: Vec<&'s str>,
    pub children: Vec<DocumentElement<'s>>,
    pub is_comment: bool,
    pub is_archived: bool,
}

#[derive(Debug)]
pub struct Section<'s> {
    pub source: &'s str,
    pub children: Vec<Element<'s>>,
}

#[derive(Debug)]
pub enum DocumentElement<'s> {
    Heading(Heading<'s>),
    Section(Section<'s>),
}

#[derive(Debug)]
pub enum TodoKeywordType {
    Todo,
    Done,
}

impl<'s> GetStandardProperties<'s> for DocumentElement<'s> {
    fn get_standard_properties(&'s self) -> &'s dyn StandardProperties {
        match self {
            DocumentElement::Heading(inner) => inner,
            DocumentElement::Section(inner) => inner,
        }
    }
}

impl<'s> StandardProperties<'s> for Document<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Section<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> StandardProperties<'s> for Heading<'s> {
    fn get_source(&'s self) -> &'s str {
        self.source
    }
}

impl<'s> Document<'s> {
    simple_getter!(get_category, category, Option<&'s str>);
    ref_getter!(get_zeroth_section, zeroth_section, Option<Section<'s>>);
    ref_getter!(get_children, children, Vec<Heading<'s>>);
}
